/*
 * Copyright © 2021-2023 Fachhochschule für die Wirtschaft (FHDW) Hannover
 *
 * This file is part of ipspiel24-GD.
 *
 * ipspiel24-GD is free software: you can redistribute it and/or modify it under the terms of the GNU General Public
 * License as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * ipspiel24-GD is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along with ipspiel24-GD. If not, see
 * <http://www.gnu.org/licenses/>.
 */
package de.fhdw.gaming.GefangenenDilemma.strategy;

import java.util.Optional;
import java.util.List;
import java.util.Map.Entry;

import de.fhdw.gaming.GefangenenDilemma.domain.GDPlayer;
import de.fhdw.gaming.GefangenenDilemma.domain.GDState;
import de.fhdw.gaming.GefangenenDilemma.domain.GDStrategy;
import de.fhdw.gaming.GefangenenDilemma.moves.GDMove;
import de.fhdw.gaming.GefangenenDilemma.moves.factory.GDMoveFactory;
import de.fhdw.gaming.GefangenenDilemma.moves.impl.GDRemainSilentMove;
import de.fhdw.gaming.GefangenenDilemma.moves.impl.GDSnitchMove;
import de.fhdw.gaming.core.domain.Move;
import de.fhdw.gaming.memory.Key;
import de.fhdw.gaming.memory.impl.MemoryBrainImpl;
import de.fhdw.gaming.memory.impl.MemoryKeyImpl;
import de.fhdw.gaming.memory.impl.MemoryPairImpl;

/**
 * Implements {@link GDStrategy} by always saying "yes".
 */
public final class GDTitForTatStrategy implements GDStrategy {

    /**
     * The factory for creating Gefangenen-Dilemma moves.
     */
    private final GDMoveFactory moveFactory;

    /**
     * Creates an {@link GDTitForTatStrategy}.
     *
     * @param moveFactory The factory for creating Gefangenen-Dilemma moves.
     */
    GDTitForTatStrategy(final GDMoveFactory moveFactory) {
        this.moveFactory = moveFactory;
    }

    @Override
    public Optional<GDMove> computeNextMove(
            final int gameId,
            final GDPlayer player,
            final GDState state,
            final long maxComputationTimePerMove) {
        
        //MemoryBrainImpl.getInstance().setMemoryLength(1);
        
        GDPlayer opponent = null;
        for (Entry<String, GDPlayer> entry : state.getPlayers().entrySet()) {
            if (!entry.getValue().equals(player)) {
                opponent = entry.getValue();
                }
        }
        
        final Key key = new MemoryKeyImpl(player, opponent, this);
        
        final List<MemoryPairImpl> check = MemoryBrainImpl.getInstance().getMemory(key).show();
        
        //Add code inside this if
        if (check != null && !check.isEmpty()) {
            final Move<?, ?> lastMoveUsed =
                    MemoryBrainImpl.getInstance().getMemory(key).show().get(0).getOpponentMove();
            
            
            if (lastMoveUsed != null) {
                if (lastMoveUsed.getClass().equals(GDRemainSilentMove.class)) {
                    return Optional.of(this.moveFactory.createRemainSilentMove());
                } else if (lastMoveUsed.getClass().equals(GDSnitchMove.class)) {
                    return Optional.of(this.moveFactory.createSnitchMove());
                } else {
                    return Optional.of(this.moveFactory.createRandomMove());
                }
            } else {
                return Optional.of(this.moveFactory.createRandomMove());
            }
        } else {
            return Optional.of(this.moveFactory.createRandomMove());
        }


    }

    @Override
    public String toString() {
        return GDTitForTatStrategy.class.getSimpleName();
    }
}
